import { Injectable, Inject } from '@angular/core';
import { HttpClient } from '@angular/common/http';

import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

import { _MODULE_CONFIG } from '../types/token';
import { ModuleConfig } from '../types/module-config';
import {
  CheckUserInfoExRequest,
  CheckUserInfoExResponse
} from '../models/check-user-info-ex';
import {
  CompanyListRequest,
  CompanyListResponse,
  Company
} from '../models/company-list';
import {
  TokenUpdateRequest,
  TokenUpdateResponse
} from '../models/token-update';
import { UrlInfoResponse, UrlInfoRequest } from '../models/url-info';

@Injectable({
  providedIn: 'root'
})
export class ExternalApiService {
  constructor(
    @Inject(_MODULE_CONFIG) private moduleConfig: ModuleConfig,
    private httpClient: HttpClient
  ) {}

  public checkUserInfoEx(
    req: CheckUserInfoExRequest
  ): Observable<CheckUserInfoExResponse> {
    return this.httpClient
      .post<any>(
        this.moduleConfig.urls.checkUserInfoEx,
        {},
        {
          params: {
            p_user_id: req.userId,
            p_comp_code: req.companyCode,
            p_user_session: req.userSession
          }
        }
      )
      .pipe(
        map(res => {
          return {
            statusCode: res.StatusCode,
            errorMessage: res.ErrorMessage,
            userId: res.UserID
          } as CheckUserInfoExResponse;
        })
      );
  }

  public companyList(req: CompanyListRequest): Observable<CompanyListResponse> {
    return this.httpClient
      .post<any>(
        this.moduleConfig.urls.companyList,
        {},
        {
          params: {
            p_user_seq: req.userSeq,
            p_app_type: req.appType,
            p_device_type: req.deviceType,
            p_token: req.token,
            p_comp_group_code: req.companyGroupCode
          }
        }
      )
      .pipe(
        map(res => {
          let companyList: Company[] | undefined;
          if (!!res.CompanyList) {
            companyList = [];
            for (const company of res.CompanyList) {
              companyList.push({
                companyCode: company.COMPANY_CODE,
                companyName: company.COMPANY_NAME,
                companyDomain: company.COMPANY_DOMAIN,
                companyConfAuthYn: company.COMPANY_CONF_AUTH,
                ucapUseYn: company.UCAP_USE_YN,
                companyTimerChatAuthYn: company.COMPANY_TIMER_CHAT_AUTH
              });
            }
          }

          return {
            statusCode: res.StatusCode,
            errorMessage: res.ErrorMessage,
            companyList
          } as CompanyListResponse;
        })
      );
  }

  public tokenUpdate(req: TokenUpdateRequest): Observable<TokenUpdateResponse> {
    return this.httpClient
      .post<any>(
        this.moduleConfig.urls.tokenUpdate,
        {},
        {
          params: {
            p_user_seq: req.userSeq,
            p_device_type: req.deviceType,
            p_token: req.token,
            p_mobile_pid: req.mobilePid,
            p_push_type: req.pushType
          }
        }
      )
      .pipe(
        map(res => {
          return {} as TokenUpdateResponse;
        })
      );
  }

  public urlInfo(req: UrlInfoRequest): Observable<UrlInfoResponse> {
    return this.httpClient
      .post<any>(
        this.moduleConfig.urls.urlInfo,
        {},
        {
          params: {
            p_device_type: req.deviceType
          }
        }
      )
      .pipe(
        map(res => {
          return {
            statusCode: res.StatusCode,
            errorMessage: res.ErrorMessage,
            portCheckUrl: res.PortCheckURL,
            messageUrl: res.MsgURL,
            messageUrl2: res.MsgURL2,
            passwordUrl: res.PasswordURL,
            planUrl: res.PlanURL,
            planUrl2: res.PlanURL2,
            vocUrl: res.VocURL,
            confUrl: res.ConfURL,
            uprApiUrl: res.UprApiURL,
            uprSvcUrl: res.UprSvcURL,
            uprDownloadUrl: res.UprDownloadURL,
            synapViewUrl: res.SynapViewURL
          } as UrlInfoResponse;
        })
      );
  }
}
