import { Injectable, Inject } from '@angular/core';
import { HttpClient } from '@angular/common/http';

import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

import { _MODULE_CONFIG } from '../types/token';
import {
  VersionInfo2Request,
  VersionInfo2Response
} from '../models/version-info2';
import { UpdateInfoRequest, UpdateInfoResponse } from '../models/update-info';
import { ModuleConfig } from '../types/module-config';
import { ParameterUtil } from '@ucap-webmessenger/api';
import { SyncMode } from '../types/sync-mode.type';

const versionInfo2Parameters = {
  userSeq: 'p_user_seq',
  deviceType: 'p_device_type',
  token: 'p_token',
  companyGroupType: 'p_comp_group_type',
  companyCode: 'p_comp_code',
  loginId: 'p_login_id'
};

const updateInfoParameters = {
  deviceType: 'p_device_type'
};

@Injectable({
  providedIn: 'root'
})
export class PublicApiService {
  constructor(
    @Inject(_MODULE_CONFIG) private moduleConfig: ModuleConfig,
    private httpClient: HttpClient
  ) {}

  public versionInfo2(
    req: VersionInfo2Request
  ): Observable<VersionInfo2Response> {
    return this.httpClient
      .post<any>(
        this.moduleConfig.urls.versionInfo2,
        {},
        {
          params: ParameterUtil.make(versionInfo2Parameters, req)
        }
      )
      .pipe(
        map((res: any) => {
          return {
            statusCode: res.StatusCode,
            errorMessage: res.ErrorMessage,
            appVersion: res.AppVer,
            authIp: res.AuthIP === 'Y' ? true : false,
            downloadUrl: res.DownloadURL,
            fileAllowSize: Number(res.FileAllowSize),
            fileTerm: Number(res.FileTerm),
            installUrl: res.InstallURL,
            launcherAppVersion: res.LauncherAppVer,
            launcherInstallUrl: res.LauncherInstallURL,
            profileRoot: res.ProfileRoot,
            profileUploadUrl: res.ProfileUploadURL,
            protocolCode: Number(res.ProtocolCD),
            serverIp: res.ServerIP,
            syncMode: res.SyncMode as SyncMode,
            uploadUrl: res.UploadURL
          } as VersionInfo2Response;
        })
      );
  }

  public updateInfo(req: UpdateInfoRequest): Observable<UpdateInfoResponse> {
    return this.httpClient
      .post<any>(
        this.moduleConfig.urls.updateInfo,
        {},
        {
          params: ParameterUtil.make(updateInfoParameters, req)
        }
      )
      .pipe(
        map(res => {
          return {
            statusCode: res.StatusCode,
            errorMessage: res.ErrorMessage,
            appVersion: res.AppVer,
            installUrl: res.InstallURL,
            launcherAppVersion: res.LauncherAppVer,
            launcherInstallUrl: res.LauncherInstallURL
          } as UpdateInfoResponse;
        })
      );
  }
}
