import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import { of } from 'rxjs';
import { catchError, exhaustMap, map, tap } from 'rxjs/operators';

import { PublicApiService } from '@ucap-webmessenger/api-public';
import { StatusCode } from '@ucap-webmessenger/api';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';

import { postLoginSuccess } from '../../account/authentication';

import { fetch, fetchSuccess, fetchFailure } from './actions';
import { LoginInfo, KEY_LOGIN_INFO } from '../../../types';

@Injectable()
export class Effects {
  init$ = createEffect(() =>
    this.actions$.pipe(
      ofType(postLoginSuccess),
      map(action => {
        const loginInfo = this.sessionStorageService.get<LoginInfo>(
          KEY_LOGIN_INFO
        );

        return fetch({
          deviceType: loginInfo.deviceType,
          companyGroupType: 'C',
          companyCode: loginInfo.companyCode,
          loginId: loginInfo.loginId
        });
      })
    )
  );

  fetch$ = createEffect(() =>
    this.actions$.pipe(
      ofType(fetch),
      map(action => action),
      exhaustMap(req =>
        this.publicApiService.versionInfo2(req).pipe(
          map(res => {
            if (res.statusCode === StatusCode.Success) {
              console.log('fetchSuccess', res);
              return fetchSuccess(res);
            } else {
              return fetchFailure({ error: 'Failed' });
            }
          }),
          catchError(error => of(fetchFailure({ error })))
        )
      )
    )
  );

  fetchSuccess$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(fetchSuccess),
        tap(params => {})
      ),
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private publicApiService: PublicApiService,
    private sessionStorageService: SessionStorageService,
    private router: Router
  ) {}
}
