import { Injectable, Inject } from '@angular/core';
import { HttpClient } from '@angular/common/http';

import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

import { _MODULE_CONFIG } from '../types/token';
import { ModuleConfig } from '../types/module-config';
import { Login2Request, Login2Response } from '../models/login2';

@Injectable({
  providedIn: 'root'
})
export class PiService {
  constructor(
    @Inject(_MODULE_CONFIG) private moduleConfig: ModuleConfig,
    private httpClient: HttpClient
  ) {}

  public login2(req: Login2Request): Observable<Login2Response> {
    return this.httpClient
      .post<any>(
        this.moduleConfig.urls.login2,
        {},
        {
          params: {
            companyCd: req.companyCode,
            loginId: req.loginId,
            loginPw: req.loginPw
          }
        }
      )
      .pipe(
        map(res => {
          return {
            status: res.STATUS,
            version: res.VER,
            passwordHc: res.PWD_HC,
            personalInfoAgreeYn: res.PER_INFO_AGREE_YN,
            companyCode: res.COMPANY_CODE,
            userId: res.USER_ID,
            userName: res.USER_NAME,
            userSeq: res.USER_SEQ
          } as Login2Response;
        })
      );
  }
}
